@@ -1,5 +1,4 @@
 {-|
-Module      : Main
 Description : Entry-point of executable
 Copyright   : (c) Eric Mertens, 2016
 License     : ISC
@@ -7,10 +6,8 @@ Maintainer  : emertens@gmail.com
 
 Entry point into glirc2. This module sets up VTY and launches the client.
 -}
-
 module Main where
 
-import Control.Concurrent
 import Control.Exception
 import Control.Lens
 import Control.Monad
@@ -26,14 +23,16 @@ import Client.Options
 import Client.State
 import Client.State.Focus
 
+import Exports ()
+
 -- | Main action for IRC client
 main :: IO ()
 main =
   do opts <- getOptions
-     cfg  <- loadConfiguration' (view optConfigFile opts)
-     runInUnboundThread $
-       withClientState cfg $ \st0 ->
-       withVty             $ \vty ->
+     let mbPath = view optConfigFile opts
+     cfg  <- loadConfiguration' mbPath
+     withClientState mbPath cfg $ \st0 ->
+       withVty $ \vty ->
          do st1 <- clientStartExtensions    st0
             st2 <- initialNetworkLogic opts st1
             st3 <- updateTerminalSize vty   st2
@@ -54,10 +53,10 @@ loadConfiguration' path =
        Right cfg -> return cfg
        Left (ConfigurationReadFailed e) ->
          report "Failed to open configuration:" e
-       Left (ConfigurationParseFailed path e) ->
-         report ("Failed to parse configuration file: " ++ path) e
-       Left (ConfigurationMalformed path e) ->
-         report ("Malformed configuration file: " ++ path ++ "\n(try --config-format)") e
+       Left (ConfigurationParseFailed p e) ->
+         report ("Failed to parse configuration file: " ++ p) e
+       Left (ConfigurationMalformed p e) ->
+         report ("Malformed configuration file: " ++ p ++ "\n(try --config-format)") e
   where
     report problem msg =
       do hPutStrLn stderr problem
