@@ -33,6 +33,12 @@ struct glirc_message {
         size_t tags_n;
 };
 
+struct glirc_chat {
+        struct glirc_string network;
+        struct glirc_string target;
+        struct glirc_string message;
+};
+
 struct glirc_command {
         const struct glirc_string *params;
         size_t params_n;
@@ -41,6 +47,7 @@ struct glirc_command {
 typedef void *start_type         (struct glirc *G, const char *path);
 typedef void stop_type           (struct glirc *G, void *S);
 typedef enum process_result process_message_type(struct glirc *G, void *S, const struct glirc_message *);
+typedef enum process_result process_chat_type(struct glirc *G, void *S, const struct glirc_chat *);
 typedef void process_command_type(struct glirc *G, void *S, const struct glirc_command *);
 
 struct glirc_extension {
@@ -50,17 +57,28 @@ struct glirc_extension {
         stop_type            *stop;
         process_message_type *process_message;
         process_command_type *process_command;
+        process_chat_type    *process_chat;
 };
 
 int glirc_send_message(struct glirc *G, const struct glirc_message *);
-int glirc_print(struct glirc *G, enum message_code, struct glirc_string msg);
+int glirc_print(struct glirc *G, enum message_code, const char *msg, size_t msglen);
+int glirc_inject_chat(struct glirc *G,
+                const char* net, size_t netLen,
+                const char* src, size_t srcLen,
+                const char* tgt, size_t tgtLen,
+                const char* msg, size_t msgLen);
 char ** glirc_list_networks(struct glirc *G);
 char ** glirc_list_channels(struct glirc *G, struct glirc_string network);
 char ** glirc_list_channel_users(struct glirc *G, struct glirc_string network, struct glirc_string channel);
-char * glirc_my_nick(struct glirc *G, struct glirc_string network);
+void glirc_current_focus(struct glirc *G, char **net, size_t *netlen, char **tgt , size_t *tgtlen);
+char * glirc_my_nick(struct glirc *G, const char *net, size_t netlen);
 void glirc_mark_seen(struct glirc *G, struct glirc_string network, struct glirc_string channel);
 void glirc_clear_window(struct glirc *G, struct glirc_string network, struct glirc_string channel);
 int glirc_identifier_cmp(struct glirc_string s, struct glirc_string t);
+int glirc_is_channel(struct glirc *G, const char *net, size_t netlen,
+                                      const char *tgt, size_t tgtlen);
+int glirc_is_logged_on(struct glirc *G, const char *net, size_t netlen,
+                                        const char *tgt, size_t tgtlen);
 
 void glirc_free_string(char *);
 void glirc_free_strings(char **);
