@@ -1,4 +1,4 @@
-{-# Language CPP, RecordWildCards #-}
+{-# Language RecordWildCards #-}
 {-|
 Module      : Client.CApi.Exports
 Description : Foreign exports which expose functionality for extensions
@@ -12,16 +12,49 @@ of the client.
 module Client.CApi.Exports
  ( -- * Extension API types
    Glirc_send_message
+ , glirc_send_message
+
  , Glirc_print
+ , glirc_print
+
  , Glirc_list_networks
+ , glirc_list_networks
+
  , Glirc_list_channels
+ , glirc_list_channels
+
  , Glirc_list_channel_users
+ , glirc_list_channel_users
+
  , Glirc_my_nick
+ , glirc_my_nick
+
  , Glirc_identifier_cmp
+ , glirc_identifier_cmp
+
+ , Glirc_is_channel
+ , glirc_is_channel
+
+ , Glirc_is_logged_on
+ , glirc_is_logged_on
+
  , Glirc_mark_seen
+ , glirc_mark_seen
+
  , Glirc_clear_window
+ , glirc_clear_window
+
+ , Glirc_current_focus
+ , glirc_current_focus
+
  , Glirc_free_string
+ , glirc_free_string
+
  , Glirc_free_strings
+ , glirc_free_strings
+
+ , Glirc_inject_chat
+ , glirc_inject_chat
  ) where
 
 import           Client.CApi.Types
@@ -49,6 +82,7 @@ import           Foreign.Storable
 import           Irc.Identifier
 import           Irc.RawIrcMsg
 import           Irc.UserInfo
+import           Irc.Message
 import           LensUtils
 
 ------------------------------------------------------------------------
@@ -98,10 +132,6 @@ type Glirc_send_message =
   Ptr FgnMsg {- ^ pointer to message -} ->
   IO CInt    {- ^ 0 on success       -}
 
-#ifdef EXPORT_GLIRC_CAPI
-foreign export ccall "glirc_send_message" glirc_send_message :: Glirc_send_message
-#endif
-
 glirc_send_message :: Glirc_send_message
 glirc_send_message token msgPtr =
   do mvar    <- derefToken token
@@ -125,10 +155,6 @@ type Glirc_print =
   CSize   {- ^ message length    -} ->
   IO CInt {- ^ 0 on success      -}
 
-#ifdef EXPORT_GLIRC_CAPI
-foreign export ccall glirc_print :: Glirc_print
-#endif
-
 glirc_print :: Glirc_print
 glirc_print stab code msgPtr msgLen =
   do mvar <- derefToken stab
@@ -149,16 +175,46 @@ glirc_print stab code msgPtr msgLen =
 
 ------------------------------------------------------------------------
 
+-- | Print a message or error to the client window
+type Glirc_inject_chat =
+  Ptr ()  {- ^ api token         -} ->
+  CString {- ^ network           -} ->
+  CSize   {- ^ network length    -} ->
+  CString {- ^ source            -} ->
+  CSize   {- ^ source length     -} ->
+  CString {- ^ target            -} ->
+  CSize   {- ^ target length     -} ->
+  CString {- ^ message           -} ->
+  CSize   {- ^ message length    -} ->
+  IO CInt {- ^ 0 on success      -}
+
+glirc_inject_chat :: Glirc_inject_chat
+glirc_inject_chat stab netPtr netLen srcPtr srcLen tgtPtr tgtLen msgPtr msgLen =
+  do mvar <- derefToken stab
+     net  <- peekFgnStringLen (FgnStringLen netPtr netLen)
+     src  <- peekFgnStringLen (FgnStringLen srcPtr srcLen)
+     tgt  <- mkId <$> peekFgnStringLen (FgnStringLen tgtPtr tgtLen)
+     txt  <- peekFgnStringLen (FgnStringLen msgPtr msgLen)
+     now  <- getZonedTime
+
+     let msg = ClientMessage
+                 { _msgBody    = IrcBody (Privmsg (parseUserInfo src) tgt txt)
+                 , _msgTime    = now
+                 , _msgNetwork = net
+                 }
+     modifyMVar_ mvar $ \st ->
+       do return (recordChannelMessage net tgt msg st)
+     return 0
+  `catch` \SomeException{} -> return 1
+
+------------------------------------------------------------------------
+
 -- | The resulting strings and array of strings are malloc'd and the
 -- caller must free them. NULL returned on failure.
 type Glirc_list_networks =
   Ptr ()           {- ^ api token                                        -} ->
   IO (Ptr CString) {- ^ null terminated array of null terminated strings -}
 
-#ifdef EXPORT_GLIRC_CAPI
-foreign export ccall glirc_list_networks :: Glirc_list_networks
-#endif
-
 glirc_list_networks :: Glirc_list_networks
 glirc_list_networks stab =
   do mvar <- derefToken stab
@@ -180,10 +236,6 @@ type Glirc_identifier_cmp =
   CSize   {- ^ identifier 2 len -} ->
   IO CInt
 
-#ifdef EXPORT_GLIRC_CAPI
-foreign export ccall glirc_identifier_cmp :: Glirc_identifier_cmp
-#endif
-
 glirc_identifier_cmp :: Glirc_identifier_cmp
 glirc_identifier_cmp p1 n1 p2 n2 =
   do txt1 <- peekFgnStringLen (FgnStringLen p1 n1)
@@ -203,10 +255,6 @@ type Glirc_list_channels =
   CSize   {- ^ network len -} ->
   IO (Ptr CString) {- ^ null terminated array of null terminated strings -}
 
-#ifdef EXPORT_GLIRC_CAPI
-foreign export ccall glirc_list_channels :: Glirc_list_channels
-#endif
-
 glirc_list_channels :: Glirc_list_channels
 glirc_list_channels stab networkPtr networkLen =
   do mvar <- derefToken stab
@@ -230,10 +278,6 @@ type Glirc_list_channel_users =
   CSize   {- ^ channel len -} ->
   IO (Ptr CString) {- ^ null terminated array of null terminated strings -}
 
-#ifdef EXPORT_GLIRC_CAPI
-foreign export ccall glirc_list_channel_users :: Glirc_list_channel_users
-#endif
-
 glirc_list_channel_users :: Glirc_list_channel_users
 glirc_list_channel_users stab networkPtr networkLen channelPtr channelLen =
   do mvar <- derefToken stab
@@ -260,10 +304,6 @@ type Glirc_my_nick =
   CSize   {- ^ network name length -} ->
   IO CString
 
-#ifdef EXPORT_GLIRC_CAPI
-foreign export ccall glirc_my_nick :: Glirc_my_nick
-#endif
-
 glirc_my_nick :: Glirc_my_nick
 glirc_my_nick stab networkPtr networkLen =
   do mvar <- derefToken stab
@@ -287,10 +327,6 @@ type Glirc_mark_seen =
   CSize   {- ^ channel name length -} ->
   IO ()
 
-#ifdef EXPORT_GLIRC_CAPI
-foreign export ccall glirc_mark_seen :: Glirc_mark_seen
-#endif
-
 glirc_mark_seen :: Glirc_mark_seen
 glirc_mark_seen stab networkPtr networkLen channelPtr channelLen =
   do network <- peekFgnStringLen (FgnStringLen networkPtr networkLen)
@@ -318,10 +354,6 @@ type Glirc_clear_window =
   CSize   {- ^ channel name length -} ->
   IO ()
 
-#ifdef EXPORT_GLIRC_CAPI
-foreign export ccall glirc_clear_window :: Glirc_clear_window
-#endif
-
 glirc_clear_window :: Glirc_clear_window
 glirc_clear_window stab networkPtr networkLen channelPtr channelLen =
   do network <- peekFgnStringLen (FgnStringLen networkPtr networkLen)
@@ -344,10 +376,6 @@ type Glirc_free_string =
   CString {- ^ glirc allocated string -} ->
   IO ()
 
-#ifdef EXPORT_GLIRC_CAPI
-foreign export ccall glirc_free_string :: Glirc_free_string
-#endif
-
 glirc_free_string :: Glirc_free_string
 glirc_free_string = free
 
@@ -359,12 +387,91 @@ type Glirc_free_strings =
   Ptr CString {- ^ glirc allocated strings, null-terminated -} ->
   IO ()
 
-#ifdef EXPORT_GLIRC_CAPI
-foreign export ccall glirc_free_strings :: Glirc_free_strings
-#endif
-
 glirc_free_strings :: Glirc_free_strings
 glirc_free_strings p =
   unless (p == nullPtr) $
     do traverse_ free =<< peekArray0 nullPtr p
        free p
+
+------------------------------------------------------------------------
+
+-- | Free an array of heap allocated strings found as a return value
+-- from the extension API. If argument is NULL, nothing happens.
+--
+-- Free the allocated strings with @glirc_free_string@
+--
+-- Strings set to NULL if there is no current network or no
+-- current target.
+type Glirc_current_focus =
+  Ptr ()      {- ^ api token                      -} ->
+  Ptr CString {- ^ newly allocated network string -} ->
+  Ptr CSize   {- ^ network length                 -} ->
+  Ptr CString {- ^ newly allocated target string  -} ->
+  Ptr CSize   {- ^ target length                  -} ->
+  IO ()
+
+glirc_current_focus :: Glirc_current_focus
+glirc_current_focus stab netP netL tgtP tgtL =
+  do mvar <- derefToken stab
+     st   <- readMVar mvar
+     case view clientFocus st of
+       Unfocused        -> do poke' netP nullPtr
+                              poke' netL 0
+                              poke' tgtP nullPtr
+                              poke' tgtL 0
+
+       NetworkFocus n   -> do exportText netP netL n
+                              poke' tgtP nullPtr
+                              poke' tgtL 0
+
+       ChannelFocus n t -> do exportText netP netL n
+                              exportText tgtP tgtL (idText t)
+
+------------------------------------------------------------------------
+
+-- | Returns @1@ when the given target on the given network is a channel
+-- name, otherwise returns @0@.
+--
+-- If the given network is not currently active this returns @0@.
+type Glirc_is_channel =
+  Ptr ()  {- ^ api token       -} ->
+  CString {- ^ network name    -} ->
+  CSize   {- ^ network length  -} ->
+  CString {- ^ target name     -} ->
+  CSize   {- ^ target length   -} ->
+  IO CInt {- ^ boolean         -}
+
+glirc_is_channel :: Glirc_is_channel
+glirc_is_channel stab net netL tgt tgtL =
+  do mvar    <- derefToken stab
+     st      <- readMVar mvar
+     network <- peekFgnStringLen (FgnStringLen net netL)
+     target  <- peekFgnStringLen (FgnStringLen tgt tgtL)
+
+     case preview (clientConnection network) st of
+       Just cs | isChannelIdentifier cs (mkId target) -> return 1
+       _ -> return 0
+
+------------------------------------------------------------------------
+
+-- | Returns @1@ when the given target on the given network shares a
+-- channel with the user, @0@ otherwise.
+--
+-- If the given network is not currently active this returns @0@.
+type Glirc_is_logged_on =
+  Ptr ()  {- ^ api token       -} ->
+  CString {- ^ network name    -} ->
+  CSize   {- ^ network length  -} ->
+  CString {- ^ target name     -} ->
+  CSize   {- ^ target length   -} ->
+  IO CInt {- ^ boolean         -}
+
+glirc_is_logged_on :: Glirc_is_channel
+glirc_is_logged_on stab net netL tgt tgtL =
+  do mvar    <- derefToken stab
+     st      <- readMVar mvar
+     network <- peekFgnStringLen (FgnStringLen net netL)
+     target  <- peekFgnStringLen (FgnStringLen tgt tgtL)
+
+     let online = has (clientConnection network . csUsers . ix (mkId target)) st
+     return $! if online then 1 else 0
