@@ -17,22 +17,27 @@ module Client.Image.Textbox
 
 import           Client.Configuration
 import           Client.Commands
-import           Client.Commands.Arguments
+import           Client.Commands.Arguments.Renderer
+import           Client.Commands.Arguments.Parser
 import           Client.Commands.Interpolation
 import           Client.Commands.Recognizer
-import           Client.Image.Arguments
+import           Client.Image.Message
 import           Client.Image.MircFormatting
 import           Client.Image.PackedImage
 import           Client.Image.Palette
 import           Client.State
+import           Client.State.Focus
 import qualified Client.State.EditBox as Edit
 import           Control.Lens
 import           Data.Char
+import qualified Data.HashSet as HashSet
+import           Data.HashSet (HashSet)
 import           Data.List
 import           Data.Semigroup
 import qualified Data.Text as Text
 import           Graphics.Vty.Attributes
 import qualified Graphics.Vty.Image as Vty
+import           Irc.Identifier
 
 -- | Compute the UI image for the text input box. This computes
 -- the logical cursor position on the screen to compensate for
@@ -43,7 +48,7 @@ textboxImage width st
   where
   macros = views (clientConfig . configMacros) (fmap macroSpec) st
   (txt, content) =
-     views (clientTextBox . Edit.content) (renderContent macros pal) st
+     views (clientTextBox . Edit.content) (renderContent st myNick nicks macros pal) st
 
   lineImage = unpackImage (beginning <> content <> ending)
 
@@ -72,15 +77,28 @@ textboxImage width st
   beginning = char attr '^'
   ending    = char attr '$'
 
+
+  (myNick,nicks) =
+    case view clientFocus st of
+      ChannelFocus network channel ->
+         (clientHighlightsNetwork network st,
+          HashSet.fromList (channelUserList network channel st)
+         )
+      _ -> (HashSet.empty, HashSet.empty)
+
+
 -- | Renders the whole, uncropped text box as well as the 'String'
 -- corresponding to the rendered image which can be used for computing
 -- the logical cursor position of the cropped version of the text box.
 renderContent ::
+  ClientState          {- ^ client state                          -} ->
+  HashSet Identifier   {- ^ my nicknames                          -} ->
+  HashSet Identifier   {- ^ other nicknames                       -} ->
   Recognizer MacroSpec {- ^ macro completions                     -} ->
   Palette              {- ^ palette                               -} ->
   Edit.Content         {- ^ content                               -} ->
   (String, Image')     {- ^ plain text rendering, image rendering -}
-renderContent macros pal c = (txt, wholeImg)
+renderContent st myNick nicks macros pal c = (txt, wholeImg)
   where
   as  = reverse (view Edit.above c)
   bs  = view Edit.below c
@@ -92,11 +110,13 @@ renderContent macros pal c = (txt, wholeImg)
   -- ["one","two"] "three" --> "two one three"
   txt = foldl (\acc x -> x ++ ' ' : acc) leftCur as
 
+  rndr = renderLine st pal myNick nicks macros
+
   wholeImg = mconcat
            $ intersperse (plainText "\n")
-           $ map renderOtherLine as
-          ++ renderLine macros pal curTxt
-           : map renderOtherLine bs
+           $ map (rndr False) as
+          ++ rndr True curTxt
+           : map (rndr False) bs
 
 
 -- | Version of 'wcwidth' that accounts for how control characters are
@@ -112,18 +132,22 @@ myWcswidth :: String -> Int
 myWcswidth = sum . map myWcwidth
 
 
--- | Render an unfocused line
-renderOtherLine :: String -> Image'
-renderOtherLine = parseIrcTextExplicit . Text.pack
-
 -- | Render the active text box line using command highlighting and
 -- placeholders, and WYSIWYG mIRC formatting control characters.
-renderLine :: Recognizer MacroSpec -> Palette -> String -> Image'
-renderLine macros pal ('/':xs) =
+renderLine ::
+  ClientState ->
+  Palette ->
+  HashSet Identifier ->
+  HashSet Identifier ->
+  Recognizer MacroSpec {- ^ commands     -} ->
+  Bool                 {- ^ focused      -} ->
+  String               {- ^ input text   -} ->
+  Image'               {- ^ output image -}
+renderLine st pal myNick nicks macros focused ('/':xs) =
   char defAttr '/' <> string attr cmd <> continue rest
   where
     specAttr spec =
-      case parseArguments spec rest of
+      case parse st spec rest of
         Nothing -> view palCommand      pal
         Just{}  -> view palCommandReady pal
 
@@ -133,19 +157,19 @@ renderLine macros pal ('/':xs) =
       = case recognize (Text.pack cmd) allCommands of
           Exact (Right Command{cmdArgumentSpec = spec}) ->
             ( specAttr spec
-            , argumentsImage pal spec
+            , render pal st focused spec
             )
           Exact (Left (MacroSpec spec)) ->
             ( specAttr spec
-            , argumentsImage pal spec
+            , render pal st focused spec
             )
           Prefix _ ->
             ( view palCommandPrefix pal
-            , renderOtherLine
+            , parseIrcTextWithNicks pal myNick nicks focused . Text.pack
             )
           Invalid ->
             ( view palCommandError pal
-            , renderOtherLine
+            , parseIrcTextWithNicks pal myNick nicks focused . Text.pack
             )
 
-renderLine _ _ xs = parseIrcTextExplicit (Text.pack xs)
+renderLine _ pal myNick nicks _ focused xs = parseIrcTextWithNicks pal myNick nicks focused (Text.pack xs)
