@@ -31,7 +31,6 @@ module Client.State
   , clientDetailView
   , clientActivityBar
   , clientSubfocus
-  , clientNextConnectionId
   , clientNetworkMap
   , clientIgnores
   , clientConnection
@@ -43,6 +42,7 @@ module Client.State
   , clientErrorMsg
   , clientLayout
   , clientRtsStats
+  , clientConfigPath
 
   -- * Client operations
   , withClientState
@@ -53,6 +53,8 @@ module Client.State
   , clientMatcher'
   , clientActiveRegex
   , clientToggleHideMeta
+  , clientHighlightsNetwork
+  , channelUserList
 
   , consumeInput
   , currentCompletionList
@@ -129,6 +131,7 @@ import qualified Data.HashMap.Strict as HashMap
 import           Data.HashSet (HashSet)
 import qualified Data.HashSet as HashSet
 import           Data.IntMap (IntMap)
+import qualified Data.IntMap as IntMap
 import           Data.List
 import           Data.Maybe
 import           Data.Map (Map)
@@ -159,11 +162,11 @@ data ClientState = ClientState
   , _clientExtraFocus        :: ![Focus]            -- ^ extra messages windows to view
 
   , _clientConnections       :: !(IntMap NetworkState) -- ^ state of active connections
-  , _clientNextConnectionId  :: !NetworkId              -- ^ next available 'NetworkId'
   , _clientEvents            :: !(TQueue NetworkEvent)    -- ^ incoming network event queue
   , _clientNetworkMap        :: !(HashMap Text NetworkId) -- ^ network name to connection ID
 
   , _clientConfig            :: !Configuration            -- ^ client configuration
+  , _clientConfigPath        :: !(Maybe FilePath)         -- ^ client configuration file path
 
   , _clientTextBox           :: !Edit.EditBox             -- ^ primary text box
   , _clientTextBoxOffset     :: !Int                      -- ^ size to crop from left of text box
@@ -232,8 +235,8 @@ clientLine :: ClientState -> (Int, String) {- ^ line number, line content -}
 clientLine = views (clientTextBox . Edit.line) (\(Edit.Line n t) -> (n, t))
 
 -- | Construct an initial 'ClientState' using default values.
-withClientState :: Configuration -> (ClientState -> IO a) -> IO a
-withClientState cfg k =
+withClientState :: Maybe FilePath -> Configuration -> (ClientState -> IO a) -> IO a
+withClientState cfgPath cfg k =
 
   withExtensionState $ \exts ->
 
@@ -254,12 +257,12 @@ withClientState cfg k =
         , _clientSubfocus          = FocusMessages
         , _clientExtraFocus        = []
         , _clientConfig            = cfg
+        , _clientConfigPath        = cfgPath
         , _clientScroll            = 0
         , _clientDetailView        = False
         , _clientRegex             = Nothing
         , _clientLayout            = view configLayout cfg
         , _clientActivityBar       = view configActivityBar cfg
-        , _clientNextConnectionId  = 0
         , _clientBell              = False
         , _clientExtensions        = exts
         , _clientLogQueue          = []
@@ -285,9 +288,12 @@ abortNetwork ::
 abortNetwork network st =
   case preview (clientConnection network) st of
     Nothing -> return st
-    Just cs -> do abortConnection ForcedDisconnect (view csSocket cs)
+    Just cs -> do -- cancel the network thread
+                  abortConnection ForcedDisconnect (view csSocket cs)
+                  -- unassociate this network name from this network id
                   return $! over clientNetworkMap (sans network) st
 
+
 -- | Add a message to the window associated with a given channel
 recordChannelMessage ::
   Text       {- ^ network -} ->
@@ -418,7 +424,7 @@ recordIrcMessage ::
   ClientMessage ->
   ClientState -> ClientState
 recordIrcMessage network target msg st =
-  updateTransientError msg $
+  updateTransientError (NetworkFocus network) msg $
   case target of
     TargetHidden      -> st
     TargetNetwork     -> recordNetworkMessage msg st
@@ -463,11 +469,15 @@ computeUserSigils network channel user =
 
 
 -- | Detect /error/ messages and add the message text to the transient
--- error display.
-updateTransientError :: ClientMessage -> ClientState -> ClientState
-updateTransientError msg =
+-- error display. The transient message will not be generated if the
+-- user is focused on the window where the message is going to be
+-- rendered, anyway.
+updateTransientError :: Focus -> ClientMessage -> ClientState -> ClientState
+updateTransientError destination msg st
+  | view clientFocus st == destination = st
+  | otherwise =
 
-  let err = set clientErrorMsg . Just in
+  let err e = set clientErrorMsg (Just e) st in
 
   case view msgBody msg of
     ErrorBody txt       -> err txt
@@ -476,12 +486,12 @@ updateTransientError msg =
       | let info = replyCodeInfo code
       , ErrorReply <- replyCodeType info ->
           err (Text.intercalate " " (replyCodeText info : drop 1 args))
-    _ -> id
+    _ -> st
 
 
 -- | Record a message on a network window
 recordNetworkMessage :: ClientMessage -> ClientState -> ClientState
-recordNetworkMessage msg st = updateTransientError msg
+recordNetworkMessage msg st = updateTransientError focus msg
                             $ recordWindowLine focus wl st
   where
     network    = view msgNetwork msg
@@ -696,20 +706,30 @@ addConnection attempts lastTime network st =
          settings = fromMaybe defSettings
                   $ preview (clientConfig . configServers . ix network) st
 
-     let (i,st') = st & clientNextConnectionId <+~ 1
+         i = nextAvailableKey (view clientConnections st)
+
          -- don't bother delaying on the first reconnect
          delay = 15 * max 0 (attempts - 1)
+
      c <- createConnection
             delay
             i
             settings
-            (view clientEvents st')
+            (view clientEvents st)
 
      let cs = newNetworkState i network settings c (PingConnecting attempts lastTime)
      traverse_ (sendMsg cs) (initialMessages cs)
 
      return $ set (clientNetworkMap . at network) (Just i)
-            $ set (clientConnections . at i) (Just cs) st'
+            $ set (clientConnections . at i) (Just cs) st
+
+-- | Find the first unused key in the intmap starting at 0.
+nextAvailableKey :: IntMap a -> Int
+nextAvailableKey m = foldr aux id (IntMap.keys m) 0
+  where
+    aux k next i
+      | k == i    = next (i+1)
+      | otherwise = i
 
 applyMessageToClientState ::
   ZonedTime                  {- ^ timestamp                -} ->
@@ -776,7 +796,7 @@ clientStartExtensions st =
   do let cfg = view clientConfig st
      st1        <- clientStopExtensions st
      (st2, res) <- clientPark st1 $ \ptr ->
-            traverse (try . activateExtension ptr <=< resolveConfigurationPath)
+            traverse (try . activateExtension ptr)
                      (view configExtensions cfg)
 
      let (errors, exts) = partitionEithers res
