@@ -1,3 +1,4 @@
+{-# Language OverloadedStrings #-}
 {-|
 Module      : Client.View.Palette
 Description : View current palette and to see all terminal colors
@@ -15,6 +16,7 @@ module Client.View.Palette
   ) where
 
 import           Client.Image.Palette
+import           Client.Image.MircFormatting
 import           Client.Image.PackedImage
 import           Control.Lens
 import           Data.List
@@ -26,6 +28,11 @@ digits = "0123456789ABCDEF"
 digitImage :: Char -> Image'
 digitImage d = string defAttr [' ',d,' ']
 
+decimalImage :: Int -> Image'
+decimalImage n
+  | n < 10    = string defAttr (' ':'0':show n)
+  | otherwise = string defAttr (    ' ':show n)
+
 columns :: [Image'] -> Image'
 columns = mconcat . intersperse (char defAttr ' ')
 
@@ -33,30 +40,64 @@ columns = mconcat . intersperse (char defAttr ' ')
 -- all the colors used in the current palette as well as
 -- the colors available for use in palettes.
 paletteViewLines :: Palette -> [Image']
-paletteViewLines pal =
+paletteViewLines pal = reverse $
+
+  [ "Current client palette"
+  , ""
+  , columns (paletteEntries pal)
+  , ""
+
+  , "Current client palette nick highlight colors"
+  , ""
+  , columns (nickHighlights pal)
+  , ""
+
+  , "Chat formatting colors: ^C[foreground[,background]]"
+  , ""
+  , columns ("   " : map decimalImage [0..15])
+  , columns mircColors
+  , ""
+
+  , "Available palette colors: 0x<row><col>"
+  , ""
+  , columns (map digitImage (' ':digits))
+  , columns isoColors ]
+
+  ++
+
   [ columns
   $ digitImage digit
   : [ string (withBackColor defAttr c) "   "
     | col <- [0 .. 15]
     , let c = Color240 (row * 16 + col)
     ]
-  | (digit,row) <- reverse $ take 15 $ zip (drop 1 digits) [0 ..]
-  ] ++
-  [ columns
-  $ digitImage '0'
+  | (digit,row) <- zip (drop 1 digits) [0 ..]
+  ]
+
+
+isoColors :: [Image']
+isoColors =
+  digitImage '0'
   : [ string (withBackColor defAttr (ISOColor c)) "   "
     | c <- [0..15]
     ]
 
-  , columns (map digitImage (' ':digits))
-  , mempty
-  , columns
-    [ text' (view l pal) name
-    | (name, Lens l) <- paletteMap
-    ]
-  , mempty
-  , columns
-    [ string attr "nicks"
-    | attr <- toListOf (palNicks . folded) pal
+mircColors :: [Image']
+mircColors =
+  "   "
+  : [ string (withBackColor defAttr c) "   "
+    | i <- [0..15]
+    , let Just c = mircColor i
     ]
+
+paletteEntries :: Palette -> [Image']
+paletteEntries pal =
+  [ text' (view l pal) name
+  | (name, Lens l) <- paletteMap
+  ]
+
+nickHighlights :: Palette -> [Image']
+nickHighlights pal =
+  [ string attr "nicks"
+  | attr <- toListOf (palNicks . folded) pal
   ]
